@@ -54,6 +54,7 @@ class BaseInteraction:
     def __init__(self, client, data: dict):
         state = client._connection
 
+        self.bot = client
         self.id = int(data["id"])
         self.application_id = int(data["application_id"])
         self.type = data["type"]
@@ -79,8 +80,6 @@ class BaseInteraction:
         else:
             self.channel = None
         
-        self.editable = False
-        self._client = client
         self._sent = False
         self._webhook = None
 
@@ -88,8 +87,8 @@ class BaseInteraction:
     def webhook(self):
         if self._webhook is None:
             self._webhook = discord.Webhook(
-                {"id": self._client.user.id, "type": 1, "token": self.token},
-                adapter=discord.AsyncWebhookAdapter(self._client.http._HTTPClient__session)
+                {"id": self.application_id, "type": 1, "token": self.token},
+                adapter=discord.AsyncWebhookAdapter(self.bot.http._HTTPClient__session)
             )
         return self._webhook
     
@@ -187,18 +186,16 @@ class BaseInteraction:
             allowed_mentions=allowed_mentions
         )
         self._sent = True
-        # Type-5 responses are always editable
+
         if type == 5:
-            self.editable = True
             return None
         # Ephemeral messages aren't stored and can't be deleted or edited
         # Same for type-1 and type-2 messages
         if ephemeral or type in (1, 2):
             return None
         
-        self.editable = True
         if delete_after is not None:
-            self._client.loop.create_task(self.delete_after(delete_after))
+            self.bot.loop.create_task(self.delete_after(delete_after))
         
         if fetch_response_message:
             return await self.fetch_initial_response()
@@ -260,13 +257,17 @@ class BaseInteraction:
         if components is not None:
             if len(components) > 5:
                 raise discord.InvalidArgument("components must be a list of up to 5 elements")
-            if not all(isinstance(comp, ActionRow) for comp in components):
-                raise discord.InvalidArgument("components must be a list of ActionRow")
-            data["components"] = [comp.to_dict() for comp in components]
+            wrapped = []
+            for comp in components:
+                if isinstance(comp, ActionRow):
+                    wrapped.append(comp)
+                else:
+                    wrapped.append(ActionRow(comp))
+            data["components"] = [comp.to_dict() for comp in wrapped]
         
         # Allowed mentions
         if content or embed or embeds:
-            state = self._client._connection
+            state = self.bot._connection
             if allowed_mentions is not None:
                 if state.allowed_mentions is not None:
                     allowed_mentions = state.allowed_mentions.merge(allowed_mentions).to_dict()
@@ -283,7 +284,7 @@ class BaseInteraction:
         if len(data) > 0:
             _json["data"] = data
         # HTTP-request
-        await self._client.http.request(
+        await self.bot.http.request(
             Route(
                 'POST', '/interactions/{interaction_id}/{token}/callback',
                 interaction_id=self.id, token=self.token
@@ -313,8 +314,6 @@ class BaseInteraction:
         message : :class:`discord.Message`
             The message that was edited
         """
-        if not self.editable:
-            raise TypeError("There's nothing to edit or the message is ephemeral.")
         # Form JSON params
         data = {}
         if content is not None:
@@ -342,7 +341,7 @@ class BaseInteraction:
                 raise discord.InvalidArgument("components must be a list of ActionRow")
             data["components"] = [comp.to_dict() for comp in components]
         # Allowed mentions
-        state = self._client._connection
+        state = self.bot._connection
         if allowed_mentions is not None:
             if state.allowed_mentions is not None:
                 allowed_mentions = state.allowed_mentions.merge(allowed_mentions).to_dict()
@@ -350,10 +349,10 @@ class BaseInteraction:
                 allowed_mentions = allowed_mentions.to_dict()
             data['allowed_mentions'] = allowed_mentions
         # HTTP-response
-        r = await self._client.http.request(
+        r = await self.bot.http.request(
             Route(
                 'PATCH', '/webhooks/{app_id}/{token}/messages/@original',
-                app_id=self._client.user.id, token=self.token
+                app_id=self.application_id, token=self.token
             ),
             json=data
         )
@@ -367,16 +366,12 @@ class BaseInteraction:
         """
         Deletes the original interaction response.
         """
-        if not self.editable:
-            raise TypeError("There's nothing to delete. Send a reply first.")
-        # patch
-        await self._client.http.request(
+        await self.bot.http.request(
             Route(
                 'DELETE', '/webhooks/{app_id}/{token}/messages/@original',
-                app_id=self._client.user.id, token=self.token
+                app_id=self.application_id, token=self.token
             )
         )
-        self.editable = False
     
     async def delete_after(self, delay: float):
         await asyncio.sleep(delay)
@@ -389,14 +384,14 @@ class BaseInteraction:
         """
         Fetches the original interaction response.
         """
-        data = await self._client.http.request(
+        data = await self.bot.http.request(
             Route(
                 'GET', '/webhooks/{app_id}/{token}/messages/@original',
-                app_id=self._client.user.id, token=self.token
+                app_id=self.application_id, token=self.token
             )
         )
         return discord.Message(
-            state=self._client._connection,
+            state=self.bot._connection,
             channel=self.channel,
             data=data
         )
@@ -417,7 +412,7 @@ class BaseInteraction:
             allowed_mentions=allowed_mentions
         )
         return discord.Message(
-            state=self._client._connection,
+            state=self.bot._connection,
             channel=self.channel,
             data=r
         )
