@@ -4,13 +4,13 @@ from discord.ext.commands import Context
 import asyncio
 import discord
 
-from .slash_core import SlashCommandResponse, get_class
+from .slash_core import CommandParent
 from .slash_command import SlashCommand, SlashCommandPermissions
 from .utils import ClickListener, _on_button_click
 from ._decohub import _HANDLER
 from ._send_modifications import *
 
-from ..interactions import ComponentType, SlashInteraction, MessageInteraction
+from ..interactions import ComponentType, BaseInteraction, SlashInteraction, MessageInteraction
 
 
 __all__ = ("SlashClient",)
@@ -43,10 +43,12 @@ class SlashClient:
     def __init__(self, client, *, show_warnings: bool=False, modify_send: bool=True):
         _HANDLER.client = client
         self.client = _HANDLER.client
+        self.application_id = None
         self.events = {}
         self._listeners = {}
         self._global_commands = {}
         self._guild_commands = {}
+        self._cogs_with_err_listeners = []
         self._show_warnings = show_warnings
         self._modify_send = modify_send
         self.active_shard_count = 0
@@ -99,6 +101,15 @@ class SlashClient:
                 return check(inter)
             return await self.wait_for_button_click(auto_check, timeout)
 
+        async def message_wait_for_dropdown(message, check=None, timeout=None):
+            if check is None:
+                check = lambda inter: True
+            def auto_check(inter):
+                if message.id != inter.message.id:
+                    return False
+                return check(inter)
+            return await self.wait_for_dropdown(auto_check, timeout)
+
         async def fetch_commands(guild):
             return await self.fetch_guild_commands(guild.id)
         
@@ -138,6 +149,7 @@ class SlashClient:
         Context.wait_for_button_click = ctx_wait_for_button_click
         discord.Message.create_click_listener = create_click_listener
         discord.Message.wait_for_button_click = message_wait_for_button_click
+        discord.Message.wait_for_dropdown = message_wait_for_dropdown
         discord.Guild.get_commands = get_commands
         discord.Guild.get_command = get_command
         discord.Guild.get_command_named = get_command_named
@@ -193,49 +205,47 @@ class SlashClient:
         name = func.__name__
         if name.startswith('on_'):
             name = name[3:]
-            if name in [
-                'slash_command', 'slash_command_error',
-                'ready', 'auto_register', 'button_click'
-                ]:
-                self.events[name] = func
+            self.events[name] = func
         return func
 
     def command(self, *args, **kwargs):
         """
-        A decorator that registers a function below as response for specified slash-command.
-
-        Parameters are similar to SlashCommand arguments.
-
-        If ``description`` is specified, the decorator will be interpreted as SlashCommand and
-        will be registered (or edited) automatically with the given set of arguments.
+        A decorator that allows to build a slash command.
 
         Parameters
         ----------
-
-        name : str
-            name of the slash-command you want to respond to (equals to function name by default)
-        description : str
-            if specified, the client will automatically register a command with this description
-        options : List[Option]
-            if specified, the client will
-            automatically register a command with this list of options. Requires ``description``
+        auto_sync : :class:`bool`
+            whether to automatically register the command or not. Defaults to ``True``
+        name : :class:`str`
+            name of the slash command you want to respond to (equals to function name by default).
+        description : :class:`str`
+            the description of the slash command. It will be visible in Discord.
+        options : :class:`List[Option]`
+            the list of slash command options. The options will be visible in Discord.
         default_permission : :class:`bool`
             whether the command is enabled by default when the app is added to a guild.
-            Requires ``description``
-        guild_ids : List[int]
+        guild_ids : :class:`List[int]`
             if specified, the client will register a command in these guilds.
-            Otherwise this command will be registered globally. Requires ``description``
+            Otherwise this command will be registered globally.
+        connectors : :class:`dict`
+            which function param states for each option. If the name
+            of an option already matches the corresponding function param,
+            you don't have to specify the connectors. Connectors template: 
+            ``{"option-name": "param_name", ...}``
         """
         def decorator(func):
             if not asyncio.iscoroutinefunction(func):
                 raise TypeError(f'<{func.__qualname__}> must be a coroutine function')
             name = kwargs.get('name', func.__name__)
-            new_func = SlashCommandResponse(
-                self.client, func, name,
-                kwargs.get('description'),
-                kwargs.get('options'),
-                kwargs.get("default_permission", True),
-                kwargs.get('guild_ids')
+            new_func = CommandParent(
+                func,
+                name=name,
+                description=kwargs.get('description'),
+                options=kwargs.get('options'),
+                default_permission=kwargs.get("default_permission", True),
+                guild_ids=kwargs.get('guild_ids'),
+                connectors=kwargs.get("connectors"),
+                auto_sync=kwargs.get("auto_sync", True)
             )
             self.commands[name] = new_func
             return new_func
@@ -353,7 +363,11 @@ class SlashClient:
         global_commands : List[SlashCommand]
         """
         data = await self.client.http.request(
-            Route('GET', '/applications/{app_id}/commands', app_id=self.client.user.id)
+            Route(
+                'GET',
+                '/applications/{application_id}/commands',
+                application_id=self.application_id
+            )
         )
         return [SlashCommand.from_dict(dat) for dat in data]
 
@@ -372,8 +386,11 @@ class SlashClient:
         guild_commands : List[SlashCommand]
         """
         data = await self.client.http.request(
-            Route('GET', '/applications/{app_id}/guilds/{guild_id}/commands',
-            app_id=self.client.user.id, guild_id=guild_id)
+            Route(
+                'GET', '/applications/{application_id}/guilds/{guild_id}/commands',
+                application_id=self.application_id,
+                guild_id=guild_id
+            )
         )
         return [SlashCommand.from_dict(dat) for dat in data]
     
@@ -394,8 +411,8 @@ class SlashClient:
         data = await self.client.http.request(
             Route(
                 "GET",
-                "/applications/{app_id}/commands/{cmd_id}",
-                app_id=self.client.user.id,
+                "/applications/{application_id}/commands/{cmd_id}",
+                application_id=self.application_id,
                 cmd_id=command_id
             )
         )
@@ -420,8 +437,8 @@ class SlashClient:
         data = await self.client.http.request(
             Route(
                 "GET",
-                "/applications/{app_id}/guilds/{guild_id}/commands/{cmd_id}",
-                app_id=self.client.user.id,
+                "/applications/{application_id}/guilds/{guild_id}/commands/{cmd_id}",
+                application_id=self.application_id,
                 guild_id=guild_id,
                 cmd_id=command_id
             )
@@ -444,8 +461,8 @@ class SlashClient:
         r = await self.client.http.request(
             Route(
                 "POST",
-                "/applications/{app_id}/commands",
-                app_id=self.client.user.id
+                "/applications/{application_id}/commands",
+                application_id=self.application_id
             ),
             json=slash_command.to_dict()
         )
@@ -472,7 +489,7 @@ class SlashClient:
             Route(
                 "POST",
                 "/applications/{app_id}/guilds/{guild_id}/commands",
-                app_id=self.client.user.id,
+                app_id=self.application_id,
                 guild_id=guild_id
             ),
             json=slash_command.to_dict()
@@ -498,7 +515,7 @@ class SlashClient:
             Route(
                 "PUT",
                 "/applications/{app_id}/commands",
-                app_id=self.client.user.id
+                app_id=self.application_id
             ),
             json=[sc.to_dict() for sc in slash_commands]
         )
@@ -524,7 +541,7 @@ class SlashClient:
             Route(
                 "PUT",
                 "/applications/{app_id}/guilds/{guild_id}/commands",
-                app_id=self.client.user.id,
+                app_id=self.application_id,
                 guild_id=guild_id
             ),
             json=[sc.to_dict() for sc in slash_commands]
@@ -550,7 +567,7 @@ class SlashClient:
             Route(
                 "PATCH",
                 "/applications/{app_id}/commands/{cmd_id}",
-                app_id=self.client.user.id,
+                app_id=self.application_id,
                 cmd_id=command_id
             ),
             json=slash_command.to_dict(hide_name=ignore_name)
@@ -578,7 +595,7 @@ class SlashClient:
             Route(
                 "PATCH",
                 "/applications/{app_id}/guilds/{guild_id}/commands/{cmd_id}",
-                app_id=self.client.user.id,
+                app_id=self.application_id,
                 guild_id=guild_id,
                 cmd_id=command_id
             ),
@@ -602,7 +619,7 @@ class SlashClient:
             Route(
                 "DELETE",
                 "/applications/{app_id}/commands/{cmd_id}",
-                app_id=self.client.user.id,
+                app_id=self.application_id,
                 cmd_id=command_id
             )
         )
@@ -624,7 +641,7 @@ class SlashClient:
             Route(
                 "DELETE",
                 "/applications/{app_id}/guilds/{guild_id}/commands/{cmd_id}",
-                app_id=self.client.user.id,
+                app_id=self.application_id,
                 guild_id=guild_id,
                 cmd_id=command_id
             )
@@ -668,7 +685,7 @@ class SlashClient:
             Route(
                 "GET",
                 "/applications/{app_id}/guilds/{guild_id}/commands/{command_id}/permissions",
-                app_id=self.client.user.id,
+                app_id=self.application_id,
                 guild_id=guild_id,
                 command_id=command_id
             )
@@ -688,7 +705,7 @@ class SlashClient:
             Route(
                 "GET",
                 "/applications/{app_id}/guilds/{guild_id}/commands/permissions",
-                app_id=self.client.user.id,
+                app_id=self.application_id,
                 guild_id=guild_id
             )
         )
@@ -716,7 +733,7 @@ class SlashClient:
             Route(
                 "PUT",
                 "/applications/{app_id}/guilds/{guild_id}/commands/{command_id}/permissions",
-                app_id=self.client.user.id,
+                app_id=self.application_id,
                 guild_id=guild_id,
                 command_id=command_id
             ),
@@ -749,7 +766,7 @@ class SlashClient:
             Route(
                 "PUT",
                 "/applications/{app_id}/guilds/{guild_id}/commands/permissions",
-                app_id=self.client.user.id,
+                app_id=self.application_id,
                 guild_id=guild_id
             ),
             json=data
@@ -858,6 +875,9 @@ class SlashClient:
             await self.delete_guild_command(guild_id, cmd.id)
 
     # Internal things
+    def dispatch(self, event_name, *args, **kwargs):
+        self.client.loop.create_task(self._activate_event(event_name, *args, **kwargs))
+
     def _add_global_command(self, command):
         self._global_commands[command.id] = command
 
@@ -883,14 +903,34 @@ class SlashClient:
             if command_id in granula:
                 granula[command_id].permissions = permissions
 
+    def _error_handler_exists(self):
+        return not (
+            len(self._cogs_with_err_listeners) == 0 and
+            not hasattr(self.client, "on_slash_command_error") and
+            "slash_command_error" not in self.client._listeners and
+            "slash_command_error" not in self._listeners and
+            "slash_command_error" not in self.events
+        )
+
     def _inject_cogs(self, cog):
         for cmd in self.commands.values():
             if cmd._cog_class_name == cog.__class__.__name__:
                 cmd._inject_cog(cog)
         if self.is_ready:
             self.client.loop.create_task(self._auto_register_or_patch())
+        # We need to know which cogs are able to dispatch errors
+        pairs = cog.get_listeners()
+        for name, func in pairs:
+            if name == "on_slash_command_error":
+                self._cogs_with_err_listeners.append(cog.qualified_name)
+                break
     
     def _eject_cogs(self, name):
+        try:
+            self._cogs_with_err_listeners.remove(name)
+        except Exception:
+            pass
+        # Remove the commands from cache
         bad_keys = []
         for kw, cmd in _HANDLER.commands.items():
             if cmd._cog_name == name:
@@ -909,6 +949,8 @@ class SlashClient:
         global_cmds = []
         guilds = {}
         for cmd in _HANDLER.commands.values():
+            if not cmd.auto_sync:
+                continue
             if cmd.guild_ids is None:
                 global_cmds.append(cmd.registerable)
             else:
@@ -1001,6 +1043,15 @@ class SlashClient:
         listeners.append((future, check))
         return await asyncio.wait_for(future, timeout)
 
+    async def wait_for_dropdown(self, check=None, timeout=None):
+        if check is None:
+            check = lambda ctx: True
+        future = self.client.loop.create_future()
+        if "dropdown" not in self._listeners:
+            self._listeners["dropdown"] = []
+        listeners = self._listeners["dropdown"]
+        listeners.append((future, check))
+        return await asyncio.wait_for(future, timeout)
 
     # Adding relevant listeners
     async def _on_socket_response(self, payload):
@@ -1011,12 +1062,14 @@ class SlashClient:
     async def _on_shard_connect(self, shard_id):
         self.active_shard_count += 1
         if self.active_shard_count == 1:
+            await self._fill_app_id()
             await self._cache_global_commands()
             await self._cache_guild_commands()
             await self._auto_register_or_patch()
     
     async def _on_connect(self):
         if not isinstance(self.client, discord.AutoShardedClient):
+            await self._fill_app_id()
             await self._cache_global_commands()
             await self._cache_guild_commands()
             await self._auto_register_or_patch()
@@ -1070,34 +1123,42 @@ class SlashClient:
         if event_name != "ready":
             self.client.dispatch(event_name, *args, **kwargs)
         func = self.events.get(event_name)
-        if func is not None:
-            cog = get_class(func)
-            if cog is not None:
-                await func(cog(self.client), *args, **kwargs)
-            else:
-                await func(*args, **kwargs)
+        if func:
+            await func(*args, **kwargs)
 
     async def _process_interaction(self, payload):
         _type = payload.get("type", 1)
-        if _type == 2:
+        # Received a ping
+        if _type == 1:
+            inter = BaseInteraction(self.client, payload)
+            self.dispatch('interaction', inter)
+            await inter.create_response(type=1)
+        # Slash command invoked
+        elif _type == 2:
             inter = SlashInteraction(self.client, payload)
-            # Activate event
-            await self._activate_event('slash_command', inter)
-            # Invoke command
-            SCR = self.commands.get(inter.data.name)
-            if SCR is not None:
+            self.dispatch('interaction', inter)
+            self.dispatch('slash_command', inter)
+            slash_parent = self.commands.get(inter.data.name)
+            if slash_parent:
                 try:
-                    await SCR.invoke(inter)
+                    await slash_parent.invoke(inter)
                 except Exception as err:
+                    if not self._error_handler_exists():
+                        raise err
                     await self._activate_event('slash_command_error', inter, err)
+        # Message component clicked
         elif _type == 3:
             inter = MessageInteraction(self.client, payload)
+            self.dispatch('interaction', inter)
             if inter.component.type == ComponentType.Button:
                 await self._activate_event('button_click', inter)
             elif inter.component.type == ComponentType.SelectMenu:
-                # FIXME: naming might be different
-                await self._activate_event('select_menu_click', inter)
+                await self._activate_event('dropdown', inter)
     
+    async def _fill_app_id(self):
+        data = await self.client.http.application_info()
+        self.application_id = int(data["id"])
+
     # Aliases
     register_global_command = register_global_slash_command
     
